; 16-bit x 16-bit Division Subroutine
;
; This subroutine divides the 16-bit quantity in register
; pair DE by the 16-bit quantity in register pair BC.
; The result will be stored in register pair DE and the
; remainder in register pair BC.

.DEFINE DIV16_TEMP $C000
.DEFINE DIV16_COUNT $C002

.BANK 0
.ORG 0

.SECTION "DIVISION_16x16"

DIV16:  ld	hl, DIV16_TEMP	;load hl with symbolic address
	ld	(hl), c		;save the LS byte of the divisor
	inc	hl
	ld	(hl), b		;save the MS byte of the divisor
	inc	hl
	ld	(hl), 17	;save the divisors bit count (decimal 17)
	ld	bc, 0		;BC will store the partial dividend

_DIV16_NXTBIT:
	ld	hl, DIV16_COUNT	;Load hl with address of bit count (dec 17)
	ld	a, e		;get the LS byte of the divisor
	rla			;rotate the MSB into carry
	ld	e, a
	ld	a, d		;get the MS byte of the divisor
	rla			;rotate the MSB into carry
	ld	d, a
	dec	(hl)		;decrement the bit count
	ret	z		;return if zero
	ld	a, c		;rotate the MSB of the dividend
	rla			; into the partial dividend
	ld	c, a		; stored in BC
	ld	a, b
	rla
	ld	b, a
	dec	hl		;point HL to divisor in memory
	dec	hl
	ld	a, c		;get the LS byte of partial dividend
	sub	(hl)		;subtract the LS byte of the divisor
	ld	c, a
	inc	hl
	ld	a, b		;get the MS byte of partial dividend
	sbc	(hl)		;subtract with borrow the divisor
	ld	b, a
	jp	nc, _DIV16_NOADD;if carry=0, do not add divisor to
				; the result of previous subtraction
	dec	hl
	ld	a, c		;The divisor is larger than the partial
	add	(hl)		; dividend, so the divisor must be
	ld	c, a		; added to the result of the subtraction
	inc	hl		; so that the previous value of the
	ld	a, b		; partial dividend is re-established.
	adc	(hl)
	ld	b, a

_DIV16_NOADD:
	ccf			;complement the carry
	jp	_DIV16_NXTBIT

.ENDS
